import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;

public class A1ArrayList <T> implements List <T>{
	private static final int DEFAULT_CAPACITY = 10;
	transient Object[] elementData;
	private int size;
	
	public A1ArrayList() {
		this.elementData = new Object[DEFAULT_CAPACITY];
	}
	
	@Override
	public boolean add(T element) {
		if (size == elementData.length) grow(); 
		elementData[size++] = element;
		return true;
	}

	@Override
	public void add(int index, T element) {
		if (size == elementData.length) grow(); 
		System.arraycopy(elementData, index, elementData, index + 1,
				size - index);
		elementData[index] = element;
		size++;
	}

	@Override
	public void clear() {
        for (int i = 0; i < size; i++)
            elementData[i] = null;
        
        this.elementData = new Object[DEFAULT_CAPACITY];
        size = 0;
	}

	@Override
	public boolean remove(Object o) {
        if (o == null) {
            for (int index = 0; index < size; index++)
                if (elementData[index] == null) {
                    int numMoved = size - index - 1;
                    if (numMoved > 0)
                        System.arraycopy(elementData, index+1, elementData, index,
                                         numMoved);
                    elementData[--size] = null; // clear to let GC do its work
                    return true;
                }
        } else {
            for (int index = 0; index < size; index++)
                if (o.equals(elementData[index])) {
                    int numMoved = size - index - 1;
                    if (numMoved > 0)
                        System.arraycopy(elementData, index+1, elementData, index,
                                         numMoved);
                    elementData[--size] = null; // clear to let GC do its work
                    return true;
                }
        }
        if (size * 4 < elementData.length) shrink ();
        return false;
	}

	@Override
	public T remove(int index) {
		if (index >= size)
            throw new IndexOutOfBoundsException("Index: "+index+", Size: "+size);
		
        @SuppressWarnings("unchecked")
		T oldValue = (T)elementData[index];

        int numMoved = size - index - 1;
        if (numMoved > 0)
            System.arraycopy(elementData, index+1, elementData, index, numMoved);
        elementData[--size] = null; // clear to let GC do its work

        if (size * 4 < elementData.length) shrink ();
        return oldValue;
	}

	@Override
	public int size() {
		return size;
	}

	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();
        sb.append('[');
        
        for (int i = 0; i < size; i++){
            sb.append(elementData[i] == this ? "(this Collection)" : elementData[i]);
            if (i == size - 1) return sb.append(']').toString();
            sb.append(',').append(' ');
        }
        return null;
	}

	@Override
	public boolean addAll(Collection<? extends T> c) {
		throw new UnsupportedOperationException();
	}

	@Override
	public boolean addAll(int index, Collection<? extends T> c) {
		throw new UnsupportedOperationException();
	}

	@Override
	public boolean contains(Object o) {
		throw new UnsupportedOperationException();
	}

	@Override
	public boolean containsAll(Collection<?> c) {
		throw new UnsupportedOperationException();
	}

	@Override
	public T get(int index) {
		throw new UnsupportedOperationException();
	}

	@Override
	public int indexOf(Object o) {
		throw new UnsupportedOperationException();
	}

	@Override
	public boolean isEmpty() {
		throw new UnsupportedOperationException();
	}

	@Override
	public Iterator<T> iterator() {
		throw new UnsupportedOperationException();
	}

	@Override
	public int lastIndexOf(Object o) {
		throw new UnsupportedOperationException();
	}

	@Override
	public ListIterator<T> listIterator() {
		throw new UnsupportedOperationException();
	}

	@Override
	public ListIterator<T> listIterator(int index) {
		throw new UnsupportedOperationException();
	}

	@Override
	public boolean removeAll(Collection<?> c) {
		throw new UnsupportedOperationException();
	}

	@Override
	public boolean retainAll(Collection<?> c) {
		throw new UnsupportedOperationException();
	}

	@Override
	public T set(int index, T element) {
		throw new UnsupportedOperationException();
	}

	@Override
	public List<T> subList(int fromIndex, int toIndex) {
		throw new UnsupportedOperationException();
	}

	@Override
	public Object[] toArray() {
		throw new UnsupportedOperationException();
	}

	@Override
	public <T> T[] toArray(T[] a) {
		throw new UnsupportedOperationException();
	}

	private void grow(){
		int oldCapacity = elementData.length;
		int newCapacity = oldCapacity << 1;
		elementData = Arrays.copyOf(elementData, newCapacity);
		//System.out.println(oldCapacity +":" + newCapacity);
	}

	private void shrink(){
		int oldCapacity = elementData.length;
		int newCapacity = oldCapacity >> 1;
		elementData = Arrays.copyOf(elementData, newCapacity);
		//System.out.println(oldCapacity +":" + newCapacity);
	}
}
